@@ -1,27 +1,20 @@
-""" formatting.py """
-import math
-from enum import Enum, unique
-from typing import Any, Dict, Iterable, List
+from __future__ import annotations
 
+import math
+from typing import Any
+
+from .enums import ColumnSettings, RowSettings, Verbosity
 from .layer_info import LayerInfo
 
-ALL_ROW_SETTINGS = ("depth", "var_names")
 HEADER_TITLES = {
-    "kernel_size": "Kernel Shape",
-    "input_size": "Input Shape",
-    "output_size": "Output Shape",
-    "num_params": "Param #",
-    "mult_adds": "Mult-Adds",
+    ColumnSettings.KERNEL_SIZE: "Kernel Shape",
+    ColumnSettings.INPUT_SIZE: "Input Shape",
+    ColumnSettings.OUTPUT_SIZE: "Output Shape",
+    ColumnSettings.NUM_PARAMS: "Param #",
+    ColumnSettings.MULT_ADDS: "Mult-Adds",
 }
 
 
-@unique
-class Verbosity(Enum):
-    """Contains verbosity levels."""
-
-    QUIET, DEFAULT, VERBOSE = 0, 1, 2
-
-
 class FormattingOptions:
     """Class that holds information about formatting the table output."""
 
@@ -29,9 +22,9 @@ class FormattingOptions:
         self,
         max_depth: int,
         verbose: int,
-        col_names: Iterable[str],
+        col_names: tuple[ColumnSettings, ...],
         col_width: int,
-        row_settings: Iterable[str],
+        row_settings: set[RowSettings],
     ) -> None:
         self.max_depth = max_depth
         self.verbose = verbose
@@ -40,23 +33,38 @@ class FormattingOptions:
         self.row_settings = row_settings
 
         self.layer_name_width = 40
-        self.show_var_name = "var_names" in self.row_settings
-        self.show_depth = "depth" in self.row_settings
-
-    @staticmethod
-    def get_start_str(depth: int) -> str:
-        if depth == 0:
-            return ""
-        if depth == 1:
-            return "├─"
-        return "│    " * (depth - 1) + "└─"
+        self.ascii_only = RowSettings.ASCII_ONLY in self.row_settings
+        self.show_var_name = RowSettings.VAR_NAMES in self.row_settings
+        self.show_depth = RowSettings.DEPTH in self.row_settings
 
     @staticmethod
     def str_(val: Any) -> str:
         return str(val) if val else "--"
 
+    @staticmethod
+    def get_children_layers(
+        summary_list: list[LayerInfo], layer_info: LayerInfo, index: int
+    ) -> list[LayerInfo]:
+        """Fetches all of the children of a given layer."""
+        num_children = 0
+        for layer in summary_list[index + 1 :]:
+            if layer.depth <= layer_info.depth:
+                break
+            num_children += 1
+        return summary_list[index + 1 : index + 1 + num_children]
+
+    def get_start_str(self, depth: int) -> str:
+        """This function should handle all ascii/non-ascii-related characters."""
+        if depth == 0:
+            return ""
+        if depth == 1:
+            return "+ " if self.ascii_only else "├─"
+        return ("|    " if self.ascii_only else "│    ") * (depth - 1) + (
+            "+ " if self.ascii_only else "└─"
+        )
+
     def set_layer_name_width(
-        self, summary_list: List[LayerInfo], align_val: int = 5
+        self, summary_list: list[LayerInfo], align_val: int = 5
     ) -> None:
         """
         Set layer name width by taking the longest line length and rounding up to
@@ -74,7 +82,7 @@ class FormattingOptions:
         """Calculate the total width of all lines in the table."""
         return len(tuple(self.col_names)) * self.col_width + self.layer_name_width
 
-    def format_row(self, layer_name: str, row_values: Dict[str, str]) -> str:
+    def format_row(self, layer_name: str, row_values: dict[ColumnSettings, str]) -> str:
         """Get the string representation of a single layer of the model."""
         info_to_use = [row_values.get(row_type, "") for row_type in self.col_names]
         new_line = f"{layer_name:<{self.layer_name_width}} "
@@ -90,31 +98,40 @@ class FormattingOptions:
             layer_header += ":depth-idx"
         return self.format_row(f"Layer (type{layer_header})", HEADER_TITLES)
 
-    def layer_info_to_row(self, layer_info: LayerInfo, reached_max_depth: bool) -> str:
+    def layer_info_to_row(
+        self,
+        layer_info: LayerInfo,
+        reached_max_depth: bool,
+        children_layers: list[LayerInfo],
+    ) -> str:
         """Convert layer_info to string representation of a row."""
-        row_values = {
-            "kernel_size": self.str_(layer_info.kernel_size),
-            "input_size": self.str_(layer_info.input_size),
-            "output_size": self.str_(layer_info.output_size),
-            "num_params": layer_info.num_params_to_str(reached_max_depth),
-            "mult_adds": layer_info.macs_to_str(reached_max_depth),
+        values_for_row = {
+            ColumnSettings.KERNEL_SIZE: self.str_(layer_info.kernel_size),
+            ColumnSettings.INPUT_SIZE: self.str_(layer_info.input_size),
+            ColumnSettings.OUTPUT_SIZE: self.str_(layer_info.output_size),
+            ColumnSettings.NUM_PARAMS: layer_info.num_params_to_str(reached_max_depth),
+            ColumnSettings.MULT_ADDS: layer_info.macs_to_str(
+                reached_max_depth, children_layers
+            ),
         }
         start_str = self.get_start_str(layer_info.depth)
         layer_name = layer_info.get_layer_name(self.show_var_name, self.show_depth)
-        new_line = self.format_row(f"{start_str}{layer_name}", row_values)
+        new_line = self.format_row(f"{start_str}{layer_name}", values_for_row)
 
-        if self.verbose == Verbosity.VERBOSE.value:
-            for inner_name, inner_shape in layer_info.inner_layers.items():
+        if self.verbose == Verbosity.VERBOSE:
+            for inner_name, inner_layer_info in layer_info.inner_layers.items():
                 prefix = self.get_start_str(layer_info.depth + 1)
-                extra_row_values = {"kernel_size": str(inner_shape)}
-                new_line += self.format_row(f"{prefix}{inner_name}", extra_row_values)
+                new_line += self.format_row(f"{prefix}{inner_name}", inner_layer_info)
         return new_line
 
-    def layers_to_str(self, summary_list: List[LayerInfo]) -> str:
-        """Print each layer of the model using a fancy branching diagram."""
+    def layers_to_str(self, summary_list: list[LayerInfo]) -> str:
+        """
+        Print each layer of the model using a fancy branching diagram.
+        This is necessary to handle Container modules that don't have explicit parents.
+        """
         new_str = ""
-        current_hierarchy: Dict[int, LayerInfo] = {}
-        for layer_info in summary_list:
+        current_hierarchy: dict[int, LayerInfo] = {}
+        for i, layer_info in enumerate(summary_list):
             if layer_info.depth > self.max_depth:
                 continue
 
@@ -132,12 +149,15 @@ class FormattingOptions:
                     or current_hierarchy[d].module is not hierarchy[d].module
                 ):
                     new_str += self.layer_info_to_row(
-                        hierarchy[d], reached_max_depth=False
+                        hierarchy[d], reached_max_depth=False, children_layers=[]
                     )
                     current_hierarchy[d] = hierarchy[d]
 
             reached_max_depth = layer_info.depth == self.max_depth
-            new_str += self.layer_info_to_row(layer_info, reached_max_depth)
+            children_layers = self.get_children_layers(summary_list, layer_info, i)
+            new_str += self.layer_info_to_row(
+                layer_info, reached_max_depth, children_layers
+            )
             current_hierarchy[layer_info.depth] = layer_info
 
             # remove deeper hierarchy
