@@ -1,14 +1,26 @@
-""" layer_info.py """
-from typing import Any, Dict, Iterable, List, Optional, Sequence, Union
+from __future__ import annotations
+
+from typing import Any, Dict, Iterable, Sequence, Union
 
 import torch
-import torch.nn as nn
+from torch import nn
+from torch.jit import ScriptModule
+
+from .enums import ColumnSettings
 
 DETECTED_INPUT_OUTPUT_TYPES = Union[
     Sequence[Any], Dict[Any, torch.Tensor], torch.Tensor
 ]
 
 
+def rgetattr(module: torch.nn.Module, attr: str) -> torch.Tensor:
+    """Get the tensor submodule called attr from module."""
+    for attr_i in attr.split("."):
+        module = getattr(module, attr_i)
+    assert isinstance(module, torch.Tensor)
+    return module
+
+
 class LayerInfo:
     """Class that holds information about a layer module."""
 
@@ -17,41 +29,44 @@ class LayerInfo:
         var_name: str,
         module: nn.Module,
         depth: int,
-        depth_index: Optional[int] = None,
-        parent_info: Optional["LayerInfo"] = None,
+        parent_info: LayerInfo | None = None,
     ) -> None:
         # Identifying information
         self.layer_id = id(module)
         self.module = module
-        self.class_name = str(module.__class__).split(".")[-1].split("'")[0]
-        self.inner_layers: Dict[str, List[int]] = {}
+        self.class_name = (
+            str(module.original_name)
+            if isinstance(module, ScriptModule)
+            else module.__class__.__name__
+        )
+        # {layer name: {col_name: value_for_row}}
+        self.inner_layers: dict[str, dict[ColumnSettings, Any]] = {}
         self.depth = depth
-        self.depth_index = depth_index
+        self.depth_index: int | None = None  # set at the very end
         self.executed = False
         self.parent_info = parent_info
         self.var_name = var_name
-        self.leaf_layer = not any(self.module.children())
+        self.is_leaf_layer = not any(self.module.children())
 
         # Statistics
-        self.trainable = True
+        self.trainable_params = 0
         self.is_recursive = False
-        self.input_size: List[int] = []
-        self.output_size: List[int] = []
-        self.kernel_size: List[int] = []
+        self.input_size: list[int] = []
+        self.output_size: list[int] = []
+        self.kernel_size: list[int] = []
         self.num_params = 0
         self.macs = 0
-        self.calculate_num_params()
 
     def __repr__(self) -> str:
         return f"{self.class_name}: {self.depth}"
 
     @staticmethod
     def calculate_size(
-        inputs: DETECTED_INPUT_OUTPUT_TYPES, batch_dim: Optional[int]
-    ) -> List[int]:
+        inputs: DETECTED_INPUT_OUTPUT_TYPES, batch_dim: int | None
+    ) -> list[int]:
         """Set input_size or output_size using the model's inputs."""
 
-        def nested_list_size(inputs: Sequence[Any]) -> List[int]:
+        def nested_list_size(inputs: Sequence[Any]) -> list[int]:
             """Flattens nested list size."""
             if hasattr(inputs, "tensors"):
                 return nested_list_size(inputs.tensors)  # type: ignore[attr-defined]
@@ -85,7 +100,7 @@ class LayerInfo:
 
         elif isinstance(inputs, torch.Tensor):
             size = list(inputs.size())
-            if batch_dim is not None and len(size) > batch_dim:
+            if batch_dim is not None and batch_dim < len(size):
                 size[batch_dim] = 1
 
         elif isinstance(inputs, (list, tuple)):
@@ -109,25 +124,52 @@ class LayerInfo:
                 layer_name += f"-{self.depth_index}"
         return layer_name
 
+    def get_param_count(self, name: str, param: torch.Tensor) -> tuple[int, str]:
+        """
+        Get count of number of params, accounting for mask.
+
+        Masked models save parameters with the suffix "_orig" added.
+        They have a buffer ending with "_mask" which has only 0s and 1s.
+        If a mask exists, the sum of 1s in mask is number of params.
+        """
+        if name.endswith("orig"):
+            # Remove "_orig" suffix for better readability and integration
+            without_suffix = name[:-5]
+            parameter_count = int(
+                torch.sum(rgetattr(self.module, f"{without_suffix}_mask"))
+            )
+            return parameter_count, without_suffix
+        return param.nelement(), name
+
     def calculate_num_params(self) -> None:
         """
         Set num_params, trainable, inner_layers, and kernel_size
         using the module's parameters.
         """
+        name = ""
         for name, param in self.module.named_parameters():
-            self.num_params += param.nelement()
-            self.trainable &= param.requires_grad
+            cur_params, name = self.get_param_count(name, param)
 
+            self.num_params += cur_params
+            if param.requires_grad:
+                self.trainable_params += cur_params
+
+            ksize = list(param.size())
             if name == "weight":
-                ksize = list(param.size())
                 # to make [in_shape, out_shape, ksize, ksize]
                 if len(ksize) > 1:
                     ksize[0], ksize[1] = ksize[1], ksize[0]
                 self.kernel_size = ksize
 
             # RNN modules have inner weights such as weight_ih_l0
-            elif "weight" in name:
-                self.inner_layers[name] = list(param.size())
+            self.inner_layers[name] = {
+                ColumnSettings.KERNEL_SIZE: str(ksize),
+                ColumnSettings.NUM_PARAMS: f"├─{cur_params:,}",
+            }
+        if self.inner_layers:
+            self.inner_layers[name][
+                ColumnSettings.NUM_PARAMS
+            ] = f"└─{self.inner_layers[name][ColumnSettings.NUM_PARAMS][2:]}"
 
     def calculate_macs(self) -> None:
         """
@@ -137,22 +179,21 @@ class LayerInfo:
         Please note: Returned MACs is the number of MACs for the full tensor,
         i.e., taking the batch-dimension into account.
         """
-        if self.leaf_layer:
-            for name, param in self.module.named_parameters():
-                if name in ("weight", "bias"):
-                    # ignore C when calculating Mult-Adds in ConvNd
-                    if "Conv" in self.class_name:
-                        self.macs += int(
-                            param.nelement()
-                            * prod(self.output_size[:1] + self.output_size[2:])
-                        )
-                    else:
-                        self.macs += self.output_size[0] * param.nelement()
-                # RNN modules have inner weights such as weight_ih_l0
-                elif "weight" in name or "bias" in name:
-                    self.macs += prod(self.output_size[:2]) * param.nelement()
+        for name, param in self.module.named_parameters():
+            cur_params, name = self.get_param_count(name, param)
+            if name in ("weight", "bias"):
+                # ignore C when calculating Mult-Adds in ConvNd
+                if "Conv" in self.class_name:
+                    self.macs += int(
+                        cur_params * prod(self.output_size[:1] + self.output_size[2:])
+                    )
+                else:
+                    self.macs += self.output_size[0] * cur_params
+            # RNN modules have inner weights such as weight_ih_l0
+            elif "weight" in name or "bias" in name:
+                self.macs += prod(self.output_size[:2]) * cur_params
 
-    def check_recursive(self, summary_list: List["LayerInfo"]) -> None:
+    def check_recursive(self, summary_list: list[LayerInfo]) -> None:
         """
         If the current module is already-used, mark as (recursive).
         Must check before adding line to the summary.
@@ -162,24 +203,34 @@ class LayerInfo:
                 if self.layer_id == other_layer.layer_id:
                     self.is_recursive = True
 
-    def macs_to_str(self, reached_max_depth: bool) -> str:
+    def macs_to_str(
+        self, reached_max_depth: bool, children_layers: list[LayerInfo]
+    ) -> str:
         """Convert MACs to string."""
-        if self.macs > 0 and (reached_max_depth or self.leaf_layer):
+        if self.macs <= 0:
+            return "--"
+        if self.is_leaf_layer:
             return f"{self.macs:,}"
+        if reached_max_depth:
+            sum_child_macs = sum(
+                child.macs for child in children_layers if child.is_leaf_layer
+            )
+            return f"{sum_child_macs:,}"
         return "--"
 
     def num_params_to_str(self, reached_max_depth: bool) -> str:
         """Convert num_params to string."""
         if self.is_recursive:
             return "(recursive)"
-        if self.num_params > 0 and (reached_max_depth or self.leaf_layer):
+        if self.num_params > 0 and (reached_max_depth or self.is_leaf_layer):
             param_count_str = f"{self.num_params:,}"
-            return param_count_str if self.trainable else f"({param_count_str})"
+            return param_count_str if self.trainable_params else f"({param_count_str})"
         return "--"
 
 
-def prod(num_list: Union[Iterable[int], torch.Size]) -> int:
+def prod(num_list: Iterable[int] | torch.Size) -> int:
     result = 1
-    for item in num_list:
-        result *= prod(item) if isinstance(item, Iterable) else item
+    if isinstance(num_list, Iterable):
+        for item in num_list:
+            result *= prod(item) if isinstance(item, Iterable) else item
     return result
