@@ -1,38 +1,35 @@
-""" model_statistics.py """
-from typing import Any, Iterable, List, Tuple, Union
+from __future__ import annotations
 
-import torch
+from typing import Any
 
 from .formatting import FormattingOptions
 from .layer_info import LayerInfo, prod
 
-CORRECTED_INPUT_SIZE_TYPE = List[Union[Iterable[Any], torch.Size]]
-
 
 class ModelStatistics:
     """Class for storing results of the summary."""
 
     def __init__(
         self,
-        summary_list: List[LayerInfo],
-        input_size: CORRECTED_INPUT_SIZE_TYPE,
+        summary_list: list[LayerInfo],
+        input_size: Any,
+        total_input_size: int,
         formatting: FormattingOptions,
     ) -> None:
         self.summary_list = summary_list
         self.input_size = input_size
         self.formatting = formatting
+        self.total_input = total_input_size
         self.total_params, self.trainable_params = 0, 0
         self.total_output, self.total_mult_adds = 0, 0
-        self.total_input = sum(prod(sz) for sz in input_size) if input_size else 0
 
         for layer_info in summary_list:
-            if layer_info.leaf_layer:
+            if layer_info.is_leaf_layer:
                 self.total_mult_adds += layer_info.macs
                 if layer_info.is_recursive:
                     continue
                 self.total_params += layer_info.num_params
-                if layer_info.trainable:
-                    self.trainable_params += layer_info.num_params
+                self.trainable_params += layer_info.trainable_params
                 if layer_info.num_params > 0:
                     # x2 for gradients
                     self.total_output += 2 * prod(layer_info.output_size)
@@ -52,18 +49,19 @@ class ModelStatistics:
         )
         if self.input_size:
             summary_str += (
-                "Total mult-adds ({}): {:0.2f}\n{}\n"
+                "Total mult-adds ({}): {:0.2f}\n{}\n"  # pylint: disable=consider-using-f-string  # noqa
                 "Input size (MB): {:0.2f}\n"
                 "Forward/backward pass size (MB): {:0.2f}\n"
                 "Params size (MB): {:0.2f}\n"
                 "Estimated Total Size (MB): {:0.2f}\n".format(
                     *self.to_readable(self.total_mult_adds),
                     divider,
-                    self.to_bytes(self.total_input),
-                    self.to_bytes(self.total_output),
-                    self.to_bytes(self.total_params),
-                    self.to_bytes(
-                        self.total_input + self.total_output + self.total_params
+                    self.to_megabytes(self.total_input),
+                    self.float_to_megabytes(self.total_output),
+                    self.float_to_megabytes(self.total_params),
+                    (
+                        self.to_megabytes(self.total_input)
+                        + self.float_to_megabytes(self.total_output + self.total_params)
                     ),
                 )
             )
@@ -71,12 +69,17 @@ class ModelStatistics:
         return summary_str
 
     @staticmethod
-    def to_bytes(num: int) -> float:
+    def float_to_megabytes(num: int) -> float:
         """Converts a number (assume floats, 4 bytes each) to megabytes."""
         return num * 4 / 1e6
 
     @staticmethod
-    def to_readable(num: int) -> Tuple[str, float]:
+    def to_megabytes(num: int) -> float:
+        """Converts a number (assume floats, 4 bytes each) to megabytes."""
+        return num / 1e6
+
+    @staticmethod
+    def to_readable(num: int) -> tuple[str, float]:
         """Converts a number to millions, billions, or trillions."""
         if num >= 1e12:
             return "T", num / 1e12
